class M6502 : public Processor {

	public:
	M6502(Machine *);
    ~M6502();

    void interrupt(uint16);
    void NMI(void);
    void reset(void);
    void execute(void);
    void setRunTime(uint32);
    void setRegister(uint8, uint8);
    uint32 getTime(void);

    static const int IRQ=0x01;

    private:

    typedef uint8 byte;
    typedef uint16 word;

    typedef signed char offset;

	typedef union
	{
#ifdef LSB_FIRST
  		struct { byte l,h; } B;
#else
  		struct { byte h,l; } B;
#endif
  		word W;
	} pair;

	typedef struct
	{
  		byte A,P,X,Y,S;     /* CPU registers and program counter   */
  		pair PC;

  		int IPeriod,ICount; /* Set IPeriod to number of CPU cycles */
                      /* between calls to Loop6502()         */
  		byte IRequest;      /* Set to the INT_IRQ when pending IRQ */
  		byte AfterCLI;      /* Private, don't touch                */
  		int IBackup;        /* Private, don't touch                */
  		void *User;         /* Arbitrary user data (ID,RAM*,etc.)  */
  		byte TrapBadOps;    /* Set to 1 to warn of illegal opcodes */
	 	word Trap;          /* Set Trap to address to trace from   */
  		byte Trace;         /* Set Trace=1 to start tracing        */
	} M6502Regs;

	M6502Regs *R;
};
