#include "../../global.h"

PhoenixVideoHardware::PhoenixVideoHardware(Machine *hw, ScreenOutput *disp, uint8 *cr) : VideoDevice(hw, disp, cr) {

    charROM=cr;

    for(int i=0; i<0x400; i++) {

        dirty1[i]=dirty2[i]=1;
    }

    // Create the score display and background bitmaps

    panel=display->createBitmap(26*8, 3*8);
    bg=display->createBitmap(26*8, 32*8);

    scroll=palbank=0;
}

PhoenixVideoHardware::~PhoenixVideoHardware() {

    display->freeBitmap(panel);
    display->freeBitmap(bg);
}


void PhoenixVideoHardware::update(void) {

    void *theScreen = display->getScreen();

    // We have three thing to draw - the score panel, the background
    // and the foreground characters.

    for(int y = 0; y < 32; y++) {

        // Background starts at 0x4800

        uint16 pos = 0x4800+32*25+y;

        for (int x = 0; x < 26; x++) {

            // If we need to draw this character

            if(dirty1[pos-0x4800]) {

	            // Get the character value from main memory
    	    	uint8 c = machine->readMem(this, pos);

                // Banks 2 and 3 hold the background, the pallete
                // bank decides which we choose.

                display->copyBitmap(bank[2+palbank][c], bg, x*8, y*8, ScreenOutput::solid);

                dirty1[pos-0x4800]=false;
            }
            pos -= 32;
        }
    }

    // Draw the background to the screen

    display->copyWrappedBitmap(bg, theScreen, 0, 0, 0, -scroll, ScreenOutput::solid);

    // Now the score panel

    for(int y = 0; y < 3; y++) {

        uint16 pos = 0x4000+32*25+y;

        for (int x = 0; x < 26; x++) {

            if(dirty2[pos-0x4000]) {

	            uint8 c = machine->readMem(this, pos);
                display->copyBitmap(bank[0][c], panel, x*8, y*8, ScreenOutput::solid);

                dirty2[pos-0x4000]=false;
            }
            pos -= 32;
        }
    }

    display->copyBitmap(panel, theScreen, 0, 0, ScreenOutput::solid);

    // And the foreground.  These are drawn direct to the screen, with
    // transparency set.

    for(int y = 3; y < 32; y++) {

        uint16 pos = 0x4000+32*25+y;

        for (int x = 0; x < 26; x++) {

            uint8 c = machine->readMem(this, pos);

            // Character 0 is empty, so no need to draw it

            if(c) {

                display->copyBitmap(bank[0+palbank][c], theScreen, x*8, y*8, ScreenOutput::transparent);
            }
            pos -= 32;
        }
    }

    display->update();
}

void PhoenixVideoHardware::writeVideoRAM(uint16 addr, uint8 data) {

    // We have 2 dirty buffers, for foreground and background planes

	if(addr<=0x43ff)
        dirty2[addr-0x4000]=true;
    else
        dirty1[addr-0x4800]=true;
}

void PhoenixVideoHardware::createBanks(void) {

	uint8 *decoded;
    int sprite, linenumber;
    uint8 *chardata[256];
    int base=0;

    // We create an area for the sprite data to go into for
    // converting into bitmaps

    for(int i=0; i<256; i++) {

        chardata[i]=new uint8[8*8];
    }

    for(int b=0; b<2; b++) {

        // We decode each bank twice, with different palette banks values.

	    for(int p=0; p<2; p++) {

            // Step throught the whole plane

		    for(int offset=0; offset<0x0800; offset++) {

                // 8 bytes/sprite
    			sprite=offset/8;

                // Get the line number within the sprite
		        linenumber=offset%8;

                // Convert the byte at the current address to 8 data elements
		        decoded=decodePlaneData(2, offset+base, 0x0800);

                // Then write these into the sprite - remember that they
                // are rotated, so we draw them top to bottom.
        		setSpritePixels(chardata[sprite], decoded, sprite, b, p, 7-linenumber, 0, 8, down);
		    }

		    display->createSpriteBank(chardata, bank[p+(2*b)], 256, 8, 8);
	    }

        // Now decode the next bank.

	    base=0x1000;
    }

    for(int i=0; i<256; i++) {

        delete chardata[i];
    }
}


void PhoenixVideoHardware::setScrollReg(const uint8 data) {

    scroll=data;
}

void PhoenixVideoHardware::setVideoReg(const uint8 data) {

    if (palbank != ((data >> 1) & 0x01)) {

        palbank = (data >> 1) & 0x01;

        for(int i=0; i<0x400; i++) {

            dirty1[i]=dirty2[i]=1;
        }
    }
}