#define LSB_FIRST

// These need to be correct for your target platform

typedef char int8;
typedef unsigned char uint8;
typedef short int16;
typedef unsigned short uint16;
typedef int int32;
typedef unsigned int uint32;

// This is our only global function.  It is used to set up any host-specific
// data or environment which is needed before graphics, keyboard, joystick,
// mouse and audio classes can be initialized.  You may still need to alter
// the command line parsing in main(), though.

void hostInit(void);

// Class Declarations

// Core Classes

class Epp;
class Component;
class ScreenOutput;
class SynchOutput;
class AudioOutput;
class Processor;
class VideoDevice;
class Machine;
class Keyboard;
class Joystick;
class RealStick;
class KeyStick;
class Mouse;

// Specific Hardware

// - Processors

class Z80;
class M6502;

// - Video Devices

class SinclairVideo;
class InvadersVideoHardware;
class InvadersVideo;
class LunarRescueVideo;
class PhoenixVideoHardware;
class PhoenixVideo;
class PleiadsVideo;
class CentipedeVideo;

// - Audio Devices

class Pokey;

// Machine Classes

class Spectrum;
class InvadersHardware;
class Invaders;
class LunarRescue;
class PhoenixHardware;
class Phoenix;
class Pleiads;
class Centipede;

// Header Files

// Library Headers

//#include <std/exceptio.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>

// Host Specific Headers

#include <allegro.h>
#include <time.h>
#include <audio.h>
#include <sys/farptr.h>
#include <conio.h>
#include <pc.h>

// Core Headers

#include "epp.h"
#include "machine.h"

#include "host/graphic.h"
#include "host/grasync.h"
#include "host/keyboard.h"
#include "host/audio.h"
#include "host/joystick.h"
#include "host/keystick.h"
#include "host/realstik.h"
#include "host/mouse.h"

#include "compon/compon.h"
#include "compon/cpu/process.h"
#include "compon/cpu/Z80/Z80.h"
#include "compon/cpu/M6502/M6502.h"
#include "compon/video/video.h"
#include "compon/audio/pokey.h"

// Video Systems

#include "compon/video/sincvid.h"

#include "compon/video/invadvhw.h"
#include "compon/video/invadvid.h"
#include "compon/video/lrescvid.h"

#include "compon/video/phoenvhw.h"
#include "compon/video/phoenvid.h"
#include "compon/video/pleiavid.h"

#include "compon/video/centivid.h"

// Machine Headers

#include "machines/spectrum.h"

#include "machines/invadhw.h"
#include "machines/invaders.h"
#include "machines/lrescue.h"

#include "machines/phoenhw.h"
#include "machines/phoenix.h"
#include "machines/pleiads.h"
#include "machines/centiped.h"

