#include "../global.h"

AudioOutput::AudioOutput() {

    AUDIOINFO info;
    int devnum;

    Epp::log("Starting up audio system.");

    if(AInitialize()) {

        Epp::log("Unable to init audio system!");
        exit(1);
    }

    ifstream input("audio.cfg", ios::in);

    if(input) {

        input >> devnum;
        input.close();
    }
    else {

        cout << "Audio device selection\n~~~~~~~~~~~~~~~~~~~~~~\n\nThis will only happen once, and is to select your sound card.\n\nIf you make a mistake, or change cards, then delete the audio.cfg file.\n\n";
	    cout << "Select audio device:\n\n";

    	AUDIOCAPS caps;

	    for(uint8 i=0; i<AGetAudioNumDevs(); i++) {

    	    AGetAudioDevCaps(i, &caps);
    		cout << (int)i << " " << caps.szProductName << "\n";
	    }

        cout << "\n";
	    cin >> devnum;
        cout << "\n";
	   	ofstream output("audio.cfg", ios::out);
    	output << devnum;
	    output.close();

    }

    info.nDeviceId = devnum;
    info.wFormat = AUDIO_FORMAT_8BITS | AUDIO_FORMAT_STEREO;
    info.nSampleRate = 44100;

    if (AOpenAudio(&info) != AUDIO_ERROR_NONE) {

        Epp::log("Audio device detection failed.  Trying without sound...");

	    info.nDeviceId = AUDIO_DEVICE_NONE;

	    if (AOpenAudio(&info) != AUDIO_ERROR_NONE) {

            Epp::log("Unable to open audio device (even silence!)");
            exit(1);
        }
    }

    if(AOpenVoices(8)) {

        Epp::log("Unable to allocate voices.");
        exit(1);
    }
}

AudioOutput::~AudioOutput() {

    Epp::log("Shutting down audio system.");

    ACloseAudio();
}

void *AudioOutput::openChannel(const uint8 volume, const uint8 position) {

	LPHAC voice = (LPHAC)malloc(sizeof(HAC));

	if(ACreateAudioVoice(voice)) {

        free(voice);
    	Epp::log("Unable to open audio channel.");
        exit(1);
    }

    // We define volume as 0..63, not 0..255

    ASetVoiceVolume(*voice, volume/4);
    ASetVoicePanning(*voice, position);

    return voice;
}

void AudioOutput::closeChannel(void *channel) {

    ADestroyAudioVoice(*(LPHAC)channel);
	free(channel);
}

bool AudioOutput::channelActive(void *channel) {

  	int stopped;
    AGetVoiceStatus(*(LPHAC)channel, &stopped);
    return !stopped;
}

void AudioOutput::freeSample(void *wave) {

    AFreeWaveFile((LPAUDIOWAVE)wave);
	free(wave);
}

void AudioOutput::freeSynth(void *wave) {

	ADestroyAudioData((LPAUDIOWAVE)wave);
    free(wave);
}

void *AudioOutput::loadSample(char *sample) {

	LPAUDIOWAVE wave = (LPAUDIOWAVE)malloc(sizeof(LPAUDIOWAVE));

    if(ALoadWaveFile(sample, &wave, 0L)) {

        free(wave);
        Epp::log("Error loading required sample file!");
        exit(1);
    }

    return wave;
}

void *AudioOutput::createSynth(int8 *data, const uint32 length, const uint32 freq) {

   	LPAUDIOWAVE wave;

    wave = (LPAUDIOWAVE) malloc(sizeof(AUDIOWAVE));

    if (!wave) {

    	Epp::log("Unable to allocate memory for synth sample");
        exit(1);
    }

    // Create a 8-bit mono one-shot waveform object

    wave->wFormat = AUDIO_FORMAT_8BITS | AUDIO_FORMAT_MONO;
    wave->nSampleRate = freq;
    wave->dwLength = length;
    wave->dwLoopStart = wave->dwLoopEnd = 0;

    if (ACreateAudioData(wave) != AUDIO_ERROR_NONE) {

        free(wave);
        Epp::log("Unable to create waveform.");
        exit(1);
    }

    // Copy the data into the waveform object

  	memcpy(wave->lpData, data, length);

    // Upload the data to the audio DRAM local memory

    AWriteAudioData(wave, 0L, length);

    return wave;
}
