#include "../global.h"

InvadersHardware::InvadersHardware(Epp *e, ScreenOutput *disp, AudioOutput *au,
                            Keyboard *kb, Joystick *js) : Machine(e, disp, au) {

    keyboard=kb;
    joystick=js;

    playing1=playing2=0;

}

void InvadersHardware::destroyComponents(void) {

    delete cpu;
    delete vid;
}

void InvadersHardware::emulate() {

    readROMs();
    createComponents();
    vid->setupDisplay();

    // Open up six audio channels

    for (int i=0; i<6; i++) {

        channel[i]=audio->openChannel(255, 127);
    }

    // We set this to half what we actually want - see below

    cpu->setRunTime(1000000/60);

    for(;;) {

        joystick->update();

        // Run the CPU for a bit
        cpu->execute();

        // The do the first interrupt
        cpu->interrupt(0x08);

        // Run a bit more
        cpu->execute();

        // Do the second interrupt
        cpu->interrupt(0x10);

        // NOW we do a screen update

        audio->update();
        vid->update();

        if(epp->checkInterface()) {

            destroyComponents();

            for (int i=0; i<6; i++) {

                audio->stopChannel(channel[i]);
                audio->closeChannel(channel[i]);
            }

            for (int i=0; i<8; i++) {

            	audio->freeSynth(sample[i]);
            }

            return;
        }
    }
}

void InvadersHardware::reset(void) {

    cpu->reset();
}

uint8 InvadersHardware::readMem(const Component *comp, const uint16 addr) {

    return memory[addr];
}

void InvadersHardware::writeMem(const Component *comp, const uint16 addr, const uint8 data) {

    // RAM

	if (addr>=0x2000 && addr<=0x23ff) {

        memory[addr]=data;
    }
    // Video RAM

    else if (addr>=0x2400 && addr<=0x3fff) {

        if(data != memory[addr]) {

            memory[addr]=data;
            vid->writeVideoRAM(addr-0x2400, data);
        }
    }
}

uint8 InvadersHardware::in(const Component *comp, const uint16 port) {

    uint8 data=0;

    switch (port & 0xFF) {

        case 0x01:
        // Input port
        data=0x81;
        if (keyboard->keyPressed(Keyboard::k_3)) data&=0xFE;
        if (keyboard->keyPressed(Keyboard::k_2)) data|=0x02;
        if (keyboard->keyPressed(Keyboard::k_1)) data|=0x04;
        if (joystick->jfire1()) data|=0x10;
        if (joystick->jleft()) data|=0x20;
        if (joystick->jright()) data|=0x40;
        return data;
        break;

        case 0x02:
        // Dip switches
        return dip1;
        break;

        case 0x03:
        // Invaders has a hardware bitshift calculator
        return ((((bsdata1 << 8) | bsdata2) << bsamount) >> 8) & 0xff;
        break;

        default:
        break;
    }

    return 0;
}

void InvadersHardware::out(const Component *comp, const uint16 port, const uint8 data) {

    switch (port & 0xFF) {

        case 0x02:
        bsamount=data;
        break;

        case 0x04:
        bsdata2=bsdata1;
        bsdata1=data;
        break;

        case 0x03:

        if ((data&0x01) && !audio->channelActive(channel[0])) audio->playWaveform(sample[0], channel[0]);
        else if (data&~playing1&0x02) audio->playWaveform(sample[1], channel[1]);
        else if (data&~playing1&0x04) audio->playWaveform(sample[2], channel[2]);
        else if (data&~playing1&0x08) audio->playWaveform(sample[3], channel[3]);

        playing1=data;

        break;

        case 0x05:

        if (data&~playing2&0x01) audio->playWaveform(sample[4], channel[5]);
        else if (data&~playing2&0x02) audio->playWaveform(sample[5], channel[4]);
        else if (data&~playing2&0x04) audio->playWaveform(sample[6], channel[4]);
        else if (data&~playing2&0x08) audio->playWaveform(sample[7], channel[4]);
        else if ((data&0x10) && !audio->channelActive(channel[5])) audio->playWaveform(sample[8], channel[5]);

        playing2=data;

        break;

        default:
        break;
    }
}
