#include "global.h"

int main(int argc, char *argv[]) {

    Epp::resetLog();

	hostInit();

    AudioOutput *audio=0;
    Keyboard *keyboard=0;
    Joystick *joystick=0;
    ScreenOutput *display=0;
    Mouse *mouse=0;
    Machine *machine=0;
	Epp *epp=0;

    int arg=1;

    // Initial options

    bool joyflag=true;
    bool syncflag=false;

    while((arg<argc) && (*argv[arg] == '-')) {

    	if(!strcmp(argv[arg], "-nojoy")) {

        	joyflag=false;
        }
    	else if(!strcmp(argv[arg], "-synch")) {

        	syncflag=true;
        }
        else {

        	cerr << "Unknown option: " << argv[arg] << '\n';
            exit(1);
        }

        arg++;
    }

    // Set up system, based on options

    audio=new AudioOutput();
    keyboard=new Keyboard();

    // Now we have the options parsed and the keyboard handler installed,
    // print the banner and wait for a key.

   	cout << "Welcome to E++ v1.03\n~~~~~~~~~~~~~~~~~~~~\nThis program is Copyright 1997 James Sutherland.\n\nPlease read the enclosed license.txt file before using this program.\n";
    cout << "\nIf you problems with this program, first check the Epplog.txt file.\n";
    cout << "\nPress <enter> to start emulator.\n";

    do {} while (!keyboard->keyPressed(Keyboard::k_enter));

    mouse=new Mouse();

    if (joyflag) {

    	joystick = new RealStick();
    }
   	else {

    	joystick = new KeyStick(keyboard);
    }

    if (syncflag) {

    	display = new SynchOutput();
    }
    else {

    	display = new ScreenOutput();
    }

    // All the hardware objects are created, so create the E++ object

    epp = new Epp(display, audio, keyboard, joystick, mouse);

    // Start correct emulation

    if(arg!=argc) {

        // Special cases (machines which use extra parameters).
    	if (!strcmp(argv[arg], "spectrum")) {

    		epp->setMachine(new Spectrum(epp, display, audio, keyboard, joystick, argv[arg+1]));
	    }
    	else if (machine=Epp::getMachineByName(argv[arg], epp, display, audio, keyboard, joystick, mouse)) {

    		epp->setMachine(machine);
	    }
        else {

		 	delete epp;
		    delete display;
		    delete audio;
		    delete keyboard;
		    delete joystick;
			delete mouse;
        	cerr << "Unknown emulation selected: " << argv[arg] << '\n';
            exit(1);
        }
    }

    epp->start();

 	delete epp;
    delete machine;
    delete display;
    delete audio;
    delete keyboard;
    delete joystick;
	delete mouse;

    exit(0);

}

void hostInit(void) {

	if (allegro_init()) {

		cerr << "Can't init allegro!\n";
        exit(1);
    }

    ScreenClear();
    gotoxy(1,1);
}
